#include <afblib/inbuf.h>
#include <afblib/outbuf.h>
#include <stdbool.h>
#include <string.h>
#include "mxprequest.h"
#include "mxpresponse.h"
#include "mxpclient.h"
#include <stralloc.h>

#define EQUAL(sa,str) (strncmp((sa.s),(str),(sa.len)) == 0)

void mxp_init(struct mxpservice* mxp, int fd) {
   mxp->fd = fd;
   mxp->ibuf = (inbuf) {fd};
   mxp->obuf = (outbuf) {fd};
}

bool mxp_login(struct mxpservice* mxp, char* login) {
   /* send request */
   mxp_request request = {{0}};
   stralloc_copys(&request.keyword, "id");
   stralloc_copys(&request.parameter, login);
   if (!write_mxp_request(&mxp->obuf, &request)) return false;
   if (!outbuf_flush(&mxp->obuf)) return false;
   free_mxp_request(&request);
   /* receive response */
   mxp_response response = {0};
   do {
      if (!read_mxp_response(&mxp->ibuf, &response)) return false;
   } while (response.status == MXP_CONTINUATION);
   bool result = response.status == MXP_SUCCESS;
   free_mxp_response(&response);
   return result;
}

bool mxp_stat(struct mxpservice* mxp, char* sema, struct semastatus* status) {
   /* send request */
   mxp_request request = {{0}};
   stralloc_copys(&request.keyword, "stat");
   stralloc_copys(&request.parameter, sema);
   if (!write_mxp_request(&mxp->obuf, &request)) return false;
   if (!outbuf_flush(&mxp->obuf)) return false;
   free_mxp_request(&request);
   /* receive response */
   mxp_response response = {0};
   char* holder = 0;
   do {
      if (!read_mxp_response(&mxp->ibuf, &response)) return false;
      if (response.status == MXP_CONTINUATION && holder == 0) {
	 stralloc_0(&response.message);
	 holder = strdup(response.message.s);
      }
   } while (response.status == MXP_CONTINUATION);
   bool ok = response.status == MXP_SUCCESS;
   if (ok) {
      if (EQUAL(response.message, "free")) {
	 status->state = FREE;
      } else {
	 status->state = LOCKED;
      }
      status->holder = holder;
   }
   free_mxp_response(&response);
   return ok;
}

bool mxp_lock(struct mxpservice* mxp, char* sema) {
   /* send request */
   mxp_request request = {{0}};
   stralloc_copys(&request.keyword, "lock");
   stralloc_copys(&request.parameter, sema);
   if (!write_mxp_request(&mxp->obuf, &request)) return false;
   if (!outbuf_flush(&mxp->obuf)) return false;
   free_mxp_request(&request);
   /* receive response */
   mxp_response response = {0};
   do {
      if (!read_mxp_response(&mxp->ibuf, &response)) return false;
   } while (response.status == MXP_CONTINUATION);
   bool ok = response.status == MXP_SUCCESS;
   free_mxp_response(&response);
   return ok;
}

bool mxp_release(struct mxpservice* mxp, char* sema) {
   /* send request */
   mxp_request request = {{0}};
   stralloc_copys(&request.keyword, "release");
   stralloc_copys(&request.parameter, sema);
   if (!write_mxp_request(&mxp->obuf, &request)) return false;
   if (!outbuf_flush(&mxp->obuf)) return false;
   free_mxp_request(&request);
   /* receive response */
   mxp_response response = {0};
   do {
      if (!read_mxp_response(&mxp->ibuf, &response)) return false;
   } while (response.status == MXP_CONTINUATION);
   bool ok = response.status == MXP_SUCCESS;
   free_mxp_response(&response);
   return ok;
}

void mxp_hangup(struct mxpservice* mxp) {
   close(mxp->fd);
   inbuf_free(&mxp->ibuf);
   outbuf_free(&mxp->obuf);
}
